import { ɵɵdefineInjectable, Injectable, EventEmitter, Component, Output, Input, ComponentFactoryResolver, ViewChild, ViewContainerRef, NgModule, ViewEncapsulation } from '@angular/core';
import { Subject } from 'rxjs';
import { CommonModule } from '@angular/common';
import { FormsModule } from '@angular/forms';

class NgtreegridService {
    constructor() {
        this.display_data_observable = new Subject();
        this.display_data_observable$ = this.display_data_observable.asObservable();
    }
    updateDisplayDataObservable(display_data) {
        this.display_data_observable.next(display_data);
    }
    isEmpty(value) {
        return value === '' || value === undefined || value === 'undefined';
    }
    expandAll(expand_tracker) {
        for (const key in expand_tracker) {
            if (expand_tracker.hasOwnProperty(key)) {
                expand_tracker[key] = true;
            }
        }
    }
    collapseAll(expand_tracker) {
        for (const key in expand_tracker) {
            if (expand_tracker.hasOwnProperty(key)) {
                expand_tracker[key] = false;
            }
        }
        expand_tracker['data'] = true;
    }
}
NgtreegridService.ɵprov = ɵɵdefineInjectable({ factory: function NgtreegridService_Factory() { return new NgtreegridService(); }, token: NgtreegridService, providedIn: "root" });
NgtreegridService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
NgtreegridService.ctorParameters = () => [];

class Store {
    constructor(ngTreeGridService) {
        this.ngTreeGridService = ngTreeGridService;
        this.processed_data = [];
        this.raw_data = [];
        this.group_keys = {}; // Contains all group keys.
        this.processed_tree_data = {}; // Contains all group keys.
    }
    getRawData() {
        return this.raw_data;
    }
    setRawData(raw_data) {
        this.raw_data = raw_data;
    }
    getProcessedData(processed_data) {
        return this.processed_data;
    }
    setProcessedData(processed_data) {
        this.processed_data = processed_data;
        this.setDisplayData([...processed_data]);
    }
    getDisplayData() {
        return this.display_data;
    }
    setDisplayData(display_data) {
        this.display_data = display_data;
        this.ngTreeGridService.updateDisplayDataObservable(this.display_data);
    }
    filterBy(fields, search_values) {
        this.display_data = this.processed_data.filter((record) => {
            if (record.parent) {
                return true;
            }
            let found = true;
            for (let index = 0; index < fields.length; index++) {
                let field_value = record[fields[index]];
                let search_value = search_values[index];
                // If blank then continue.
                if (!search_value) {
                    continue;
                }
                if (typeof (field_value) === 'number') {
                    if (field_value !== parseInt(search_value, 10)) {
                        found = false;
                    }
                }
                else {
                    const column = this.configs.columns[index];
                    if (!column.case_sensitive_filter) {
                        field_value = field_value.toLowerCase();
                        search_value = search_value.toLowerCase();
                    }
                    if (field_value.indexOf(search_value) === -1) {
                        found = false;
                    }
                }
            }
            return found;
        });
        this.ngTreeGridService.updateDisplayDataObservable(this.display_data);
    }
    selectAll() {
        this.display_data.forEach(data => {
            data.row_selected = true;
        });
    }
    deSelectAll() {
        this.display_data.forEach(data => {
            data.row_selected = false;
        });
    }
    /**
* Find path from root and assgn grouped data
*
* @param temp_traversed_paths It is the traversed path for the current group by key.
* @param index Current index of the leaf node
* @param group_by_data Generated group by data for the current leaf node
*/
    traverseRootData(temp_traversed_paths, index, group_by_data) {
        const paths = temp_traversed_paths[index].split('.');
        let root_keys = this.processed_tree_data;
        for (let i = 0; i < paths.length - 1; i++) {
            const path = paths[i];
            root_keys = root_keys[path];
        }
        // Set in last object to keep the reference.
        root_keys[paths[paths.length - 1]] = group_by_data;
    }
    groupData(data, configs, internal_configs, edit_tracker, expand_tracker) {
        const group_by = configs.group_by;
        // It is an array of leaf nodes.
        let last_group_data = [data];
        // It represents the path to the leaf nodes.
        let traversed_paths = ['data'];
        this.processed_tree_data = { 'data': '' };
        group_by.forEach(key => {
            const temp_traversed_paths = [];
            const temp_last_group_data = [];
            let group_keys = [];
            // Number of records in traversed_paths and last_group_data are same.
            for (let index = 0; index < last_group_data.length; index++) {
                const group_data = last_group_data[index];
                const group_by_data = this.groupByKey(group_data, key);
                this.traverseRootData(traversed_paths, index, group_by_data);
                const new_group_keys = Object.keys(group_by_data);
                const traversed_group_key = traversed_paths[index];
                // Get list of grouped data for the current group by in an array.
                new_group_keys.forEach(new_group_key => {
                    // Make keys separated by dots for all group by. Example 'data.book.type'
                    temp_traversed_paths.push(traversed_group_key + '.' + new_group_key);
                    temp_last_group_data.push(group_by_data[new_group_key]);
                });
                group_keys.push(...new_group_keys);
            }
            traversed_paths = temp_traversed_paths;
            last_group_data = temp_last_group_data;
            // Remove duplicates and blanks.
            group_keys = group_keys.filter((item, pos) => {
                return !this.ngTreeGridService.isEmpty(item) && group_keys.indexOf(item) === pos;
            });
            this.group_keys[key] = group_keys;
        });
        if (internal_configs.current_sorted_column) {
            this.processData(internal_configs.current_sorted_column.sort_type, internal_configs.current_sorted_column.name, edit_tracker, expand_tracker);
        }
        else {
            this.processData(null, null, edit_tracker, expand_tracker);
        }
        this.setRawData(data);
        this.configs = configs;
    }
    groupByKey(data, group_by) {
        const group_by_data = {};
        // Make an array of group by key.
        data.forEach(item => {
            // Check if group by key is already an array or not.
            if (!group_by_data[item[group_by]]) {
                group_by_data[item[group_by]] = [];
            }
            group_by_data[item[group_by]].push(item);
        });
        return group_by_data;
    }
    processData(sort_type, sort_by, edit_tracker, expand_tracker) {
        this.processed_data = [];
        let index = 0;
        // Make recursive call to generate records.
        this.generateData(sort_type, sort_by, this.processed_tree_data, 0, null, expand_tracker);
        this.processed_data.shift();
        // Add index to all records.
        this.processed_data.forEach(data => {
            data.idx = index++;
            edit_tracker[data.idx] = false;
        });
        // Expand root so that first level shows up.
        expand_tracker['data'] = 1;
        this.setProcessedData(this.processed_data);
    }
    generateData(sort_type, sort_by, group_data, level, parent_key, expand_tracker) {
        const group_keys = Object.keys(group_data);
        group_keys.forEach(key => {
            const items = group_data[key];
            const composite_key = parent_key ? parent_key + '.' + key : key;
            // If items is not an array then it has more group by arrays. So make recursive call.
            if (!Array.isArray(items)) {
                // Create an array of children ids.
                const children = Object.keys(items);
                const children_id = [];
                children.forEach(child => {
                    // Add child id to the composite key.
                    children_id.push(composite_key + '.' + child);
                });
                this.processed_data.push({
                    parent_id: parent_key,
                    node_id: composite_key,
                    node_text: key,
                    parent: true,
                    last_parent: false,
                    children: children_id,
                    level: level
                });
                expand_tracker[composite_key] = 0;
                // Increase level to mark the level.
                this.generateData(sort_type, sort_by, items, level + 1, composite_key, expand_tracker);
            }
            else {
                // Set Parent object.
                this.processed_data.push({
                    parent_id: parent_key,
                    node_id: composite_key,
                    node_text: key,
                    parent: true,
                    last_parent: true,
                    level: level
                });
                expand_tracker[composite_key] = 0;
                // Sort Items
                if (sort_type !== null) {
                    sort_type ? items.sort((a, b) => (a[sort_by] > b[sort_by]) ? 1 : ((b[sort_by] > a[sort_by]) ? -1 : 0)) :
                        items.sort((a, b) => (a[sort_by] < b[sort_by]) ? 1 : ((b[sort_by] < a[sort_by]) ? -1 : 0));
                }
                // Set Child object.
                items.forEach(item => {
                    item.parent = false;
                    item.parent_id = composite_key;
                    this.processed_data.push(item);
                });
            }
        });
    }
    refreshDisplayData() {
        this.display_data = this.processed_data;
        this.ngTreeGridService.updateDisplayDataObservable(this.display_data);
    }
}

class NgtreegridComponent {
    constructor(ngtreegridService) {
        this.ngtreegridService = ngtreegridService;
        this.expand_tracker = {}; // Track Expand or collapse.
        this.columns = []; // Contains all column objects.
        this.edit_tracker = {}; // Track Edit options.
        this.internal_configs = {
            show_add_row: false
        };
        this.default_configs = {
            css: {
                expand_class: 'plus',
                collapse_class: 'minus',
                add_class: 'plus',
                edit_class: '',
                delete_class: '',
                save_class: '',
                cancel_class: '',
                row_selection_class: 'selected',
                header_class: '',
                parent_class: 'parent',
                row_filter_class: ''
            },
            actions: {
                edit: false,
                add: false,
                delete: false,
                resolve_edit: false,
                resolve_add: false,
                resolve_delete: false
            },
            data_loading_text: 'Loading...',
            group_by: [],
            group_by_header: [],
            action_column_width: '50px',
            filter: false,
            multi_select: false,
            group_by_width: [],
            multi_select_width: 'auto',
            row_class_function: () => true,
            row_edit_function: () => true,
            row_delete_function: () => true,
            row_select_function: () => true
        };
        this.default_column_config = {
            sorted: 0,
            sort_type: null,
            editable: false,
            hidden: false,
            filter: true,
            case_sensitive_filter: false,
            sortable: true
        };
        this.store = new Store(this.ngtreegridService);
        this.expand = new EventEmitter();
        this.collapse = new EventEmitter();
        this.cellclick = new EventEmitter();
        this.rowselect = new EventEmitter();
        this.rowdeselect = new EventEmitter();
        this.rowselectall = new EventEmitter();
        this.rowdeselectall = new EventEmitter();
        this.rowadd = new EventEmitter();
        this.rowsave = new EventEmitter();
        this.rowdelete = new EventEmitter();
    }
    ngOnChanges() {
        this.setDefaultConfigs();
        this.setColumnNames();
        // If there is no data then do nothing.
        if (!(this.data && this.data.length > 0)) {
            window.console.warn('Data should not be empty!');
            return;
        }
        this.store.groupData(this.data, this.configs, this.internal_configs, this.edit_tracker, this.expand_tracker);
    }
    setDefaultConfigs() {
        this.store.processed_data = [];
        this.store.processed_tree_data = {};
        this.configs = Object.assign({}, this.default_configs, this.configs);
        // Deep clone.
        this.configs.actions = Object.assign({}, this.default_configs.actions, this.configs.actions);
        this.configs.css = Object.assign({}, this.default_configs.css, this.configs.css);
        if (!this.configs.group_by) {
            window.console.error('group_by field is mandatory!');
        }
        else if (!Array.isArray(this.configs.group_by)) {
            this.configs.group_by = [this.configs.group_by];
        }
        // Set default header for group_by field.
        if (!this.configs.group_by_header) {
            this.configs.group_by_header = this.configs.group_by;
        }
        else {
            if (!Array.isArray(this.configs.group_by_header)) {
                this.configs.group_by_header = [this.configs.group_by_header];
            }
            if (this.configs.group_by_header.length !== this.configs.group_by.length) {
                this.configs.group_by_header = this.configs.group_by;
            }
        }
        // Set default width for group_by field.
        if (!this.configs.group_by_width) {
            this.configs.group_by_width = this.configs.group_by.map(_ => 'auto');
        }
        else {
            if (!Array.isArray(this.configs.group_by_width)) {
                this.configs.group_by_width = [this.configs.group_by_width];
            }
            if (this.configs.group_by_width.length !== this.configs.group_by.length) {
                this.configs.group_by_width = this.configs.group_by.map(_ => 'auto');
            }
        }
    }
    setColumnNames() {
        this.columns = this.configs.columns ? this.configs.columns : [];
        // If columns doesn't exist in user's object.
        if (!this.configs.columns) {
            const column_keys = Object.keys(this.data[0]);
            // Remove group by key.
            this.configs.group_by.forEach(key => {
                column_keys.splice(column_keys.indexOf(key), 1);
            });
            // Insert Header and default configuration.
            column_keys.forEach(key => {
                this.columns.push(Object.assign({ 'header': key, 'name': key }, this.default_column_config));
            });
        }
        else {
            // Insert Header and default configuration.
            for (let i = 0; i < this.columns.length; i++) {
                this.columns[i] = Object.assign({}, this.default_column_config, this.columns[i]);
            }
        }
        if (!this.configs.group_by_column) {
            this.configs.group_by_column = Object.assign({}, this.default_column_config);
        }
    }
    onExpandRow(row) {
        this.expand.emit(row);
    }
    onCollapseRow(rec) {
        this.collapse.emit(rec);
    }
    onRowSelect(row) {
        this.rowselect.emit(row);
    }
    sortColumn(column) {
        if (!column.sortable) {
            return;
        }
        // If already sorted then reverse.
        column.sort_type = column.sorted ? !column.sort_type : 1;
        column.sorted = 1;
        this.internal_configs.current_sorted_column = column;
        // Sort array.
        this.store.processData(column.sort_type, column.name, this.edit_tracker, this.expand_tracker);
    }
    addRowToGrid() {
        this.store.processed_tree_data = {};
        this.edit_tracker = {};
        this.store.groupData(this.data, this.configs, this.internal_configs, this.edit_tracker, this.expand_tracker);
    }
    onRowAdd(rec) {
        if (this.configs.actions.resolve_delete) {
            const promise = new Promise((resolve, reject) => {
                this.rowadd.emit({
                    data: rec,
                    resolve: resolve
                });
            });
            promise.then(() => {
                this.addRowToGrid();
            }).catch((err) => { });
        }
        else {
            this.addRowToGrid();
            this.rowadd.emit(rec);
        }
    }
    expandAll() {
        this.ngtreegridService.expandAll(this.expand_tracker);
    }
    collapseAll() {
        this.ngtreegridService.collapseAll(this.expand_tracker);
    }
    selectAll() {
        this.store.selectAll();
        this.internal_configs.all_selected = true;
    }
    deSelectAll() {
        this.store.deSelectAll();
        this.internal_configs.all_selected = false;
    }
}
NgtreegridComponent.decorators = [
    { type: Component, args: [{
                selector: 'db-ngtreegrid',
                template: "<table class=\"db-tree-grid\">\r\n    <thead db-tree-head\r\n      [columns]=\"columns\" \r\n      [configs]=\"configs\" \r\n      [internal_configs]=\"internal_configs\"\r\n      [store]=\"store\"\r\n      [rowselectall]=\"rowselectall\"\r\n      [rowdeselectall]=\"rowdeselectall\"\r\n      (sortcolumn)=\"sortColumn($event)\">\r\n    </thead>\r\n    \r\n    <tbody db-tree-body\r\n      [columns]=\"columns\" \r\n      [configs]=\"configs\" \r\n      [internal_configs]=\"internal_configs\"\r\n      [store]=\"store\"\r\n      [expand_tracker]=\"expand_tracker\" \r\n      [edit_tracker]=\"edit_tracker\"\r\n      [rowsave]=\"rowsave\"\r\n      [rowdelete]=\"rowdelete\"\r\n      [cellclick]=\"cellclick\"\r\n      (rowselect)=\"onRowSelect($event)\"\r\n      [rowdeselect]=\"rowdeselect\"\r\n      (rowexpand)=\"onExpandRow($event)\"\r\n      (rowcollapse)=\"onCollapseRow($event)\"\r\n      (rowadd)=\"onRowAdd($event)\">\r\n    </tbody>   \r\n    \r\n  </table>\r\n  ",
                styles: [".db-tree-grid{border:1px solid grey;border-collapse:collapse;border-spacing:0;color:#4b4b4b;display:table;line-height:1.5em;max-width:100%;overflow:auto;word-break:normal;word-break:keep-all}"]
            },] }
];
NgtreegridComponent.ctorParameters = () => [
    { type: NgtreegridService }
];
NgtreegridComponent.propDecorators = {
    expand: [{ type: Output }],
    collapse: [{ type: Output }],
    cellclick: [{ type: Output }],
    rowselect: [{ type: Output }],
    rowdeselect: [{ type: Output }],
    rowselectall: [{ type: Output }],
    rowdeselectall: [{ type: Output }],
    rowadd: [{ type: Output }],
    rowsave: [{ type: Output }],
    rowdelete: [{ type: Output }],
    data: [{ type: Input }],
    configs: [{ type: Input }]
};

class TreeBodyComponent {
    constructor() {
        this.loadingColspanCount = 0;
        this.rowadd = new EventEmitter();
        this.rowexpand = new EventEmitter();
        this.rowcollapse = new EventEmitter();
        this.rowselect = new EventEmitter();
    }
    ngOnInit() {
        this.loadingColspanCount = this.columns.length + 1;
        if (this.configs.multi_select) {
            this.loadingColspanCount++;
        }
        if (this.configs.actions.edit || this.configs.actions.add || this.configs.actions.delete) {
            this.loadingColspanCount++;
        }
    }
    fetchTraversedPaths(traversed_paths) {
        const paths = traversed_paths.split('.');
        let intermediate = this.store.processed_tree_data;
        for (let i = 0; i < paths.length; i++) {
            const path = paths[i];
            intermediate = intermediate[path];
        }
        return intermediate;
    }
    expandRow(id, row) {
        this.expand_tracker[id] = 1;
        // If children is a parent and its value is empty then expand it automatically.
        row.children && row.children.forEach(child => {
            const ids = child.split('.');
            const last_child = ids[ids.length - 1];
            // If empty!
            if (this.isEmpty(last_child)) {
                this.expand_tracker[child] = 1;
            }
        });
        this.rowexpand.emit(row);
    }
    collapseRow(id, rec) {
        this.expand_tracker[id] = 0;
        // Collapse all of its children.
        const keys = Object.keys(this.expand_tracker);
        keys.forEach(key => {
            if (key.indexOf(id) !== -1) {
                this.expand_tracker[key] = 0;
            }
        });
        this.rowcollapse.emit(rec);
    }
    isEmpty(value) {
        return value === '' || value === undefined || value === 'undefined';
    }
    range(end) {
        const array = [];
        let current = 1;
        while (current < end) {
            array.push(current);
            current += 1;
        }
        return array;
    }
    rowAdd(row) {
        this.rowadd.emit(row);
    }
    saveRecord($event) {
        const rec = $event.data;
        if (this.configs.actions.resolve_edit) {
            const promise = new Promise((resolve, reject) => {
                this.rowsave.emit({
                    data: rec,
                    resolve: resolve
                });
            });
            promise.then(() => {
                this.edit_tracker[rec['idx']] = false;
            }).catch((err) => { });
        }
        else {
            this.edit_tracker[rec['idx']] = false;
            this.rowsave.emit(rec);
        }
    }
    cancelEdit(row_data) {
        const index = row_data['idx'];
        this.edit_tracker[index] = false;
        // Cancel all changes ie copy from back up.
        Object.assign(row_data, this.internal_configs.current_edited_row);
    }
    prepareRowClass(data) {
        const row_class = { 'child': !data.parent };
        row_class[this.configs.css.parent_class] = data.parent;
        row_class[this.configs.css.row_selection_class] = data.row_selected;
        return row_class;
    }
    selectRow(row_data, event) {
        // Don't run if Multi select is enabled.
        if (this.configs.multi_select) {
            return;
        }
        if (row_data.parent) {
            return;
        }
        this.store.processed_data.forEach(data => {
            data.row_selected = false;
        });
        row_data.row_selected = true;
        this.rowselect.emit({ data: row_data, event: event });
    }
    selectRowOnCheck(row_data, event) {
        if (!row_data.row_selected) {
            row_data.row_selected = true;
            this.rowselect.emit({ data: row_data, event: event });
        }
        else {
            row_data.row_selected = false;
            this.rowdeselect.emit({ data: row_data, event: event });
        }
        this.setSelectAllConfig();
    }
    /**
     * Set Select All config on Select change.
     *
     */
    setSelectAllConfig() {
        let select_all = true;
        this.store.getDisplayData().forEach(data => {
            if (!data.row_selected) {
                select_all = false;
            }
        });
        this.internal_configs.all_selected = select_all;
    }
    showSelectBox(row_data) {
        if (this.configs.row_select_function) {
            return this.configs.row_select_function(row_data);
        }
        else {
            return true;
        }
    }
}
TreeBodyComponent.decorators = [
    { type: Component, args: [{
                selector: '[db-tree-body]',
                template: "\r\n<tr *ngIf=\"store.raw_data.length==0\">\r\n  <td [innerHTML]=\"configs.data_loading_text\" [attr.colspan]=\"loadingColspanCount\" style=\"text-align: center\"></td>\r\n</tr>\r\n<tr db-filter-row \r\n  [columns]=\"columns\" \r\n  [configs]=\"configs\"\r\n  [store]=\"store\"\r\n  [internal_configs]=\"internal_configs\" \r\n  *ngIf=\"configs.filter\"\r\n  [expand_tracker]=\"expand_tracker\"\r\n  [ngClass]=\"configs.css.row_filter_class\">\r\n</tr>\r\n<tr db-add-row \r\n  [columns]=\"columns\" \r\n  [configs]=\"configs\" \r\n  [internal_configs]=\"internal_configs\" \r\n  [store]=\"store\"\r\n  (rowadd)=\"rowAdd($event)\"\r\n  *ngIf=\"internal_configs.show_add_row\">\r\n</tr>\r\n\r\n<tr \r\n  *ngFor=\"let data of store.display_data\"\r\n  [ngClass]=\"prepareRowClass(data)\" \r\n  [attr.id]=\"data['idx']\" \r\n  (click)=\"selectRow(data, $event)\" \r\n  class=\"{{configs.row_class_function(data)}}\">\r\n\r\n  <!-- If parent text is empty then hide -->\r\n  <ng-container *ngIf=\"expand_tracker[data.parent_id] && data.parent && !isEmpty(data.node_text)\">\r\n    <td *ngIf=\"configs.multi_select\" class=\"checkbox_column\">\r\n      <!-- <input type=\"checkbox\" [checked]=\"data.row_selected\" (click)=\"selectRowOnCheck(data, $event)\"> -->\r\n    </td>\r\n    <td *ngIf=\"(configs.actions.edit || configs.actions.add || configs.actions.delete)\"></td>\r\n    <!-- Leave left td for down level parents -->\r\n    <td *ngFor=\"let level of range(data.level)\" class=\"clear-right-border clear-left-border\"></td>\r\n    <td *ngIf=\"!expand_tracker[data.node_id]\"   \r\n        [ngClass]=\"{\r\n          'clear-right-border': true, \r\n          'clear-left-border': (configs.actions.edit || configs.actions.add || configs.actions.delete)\r\n        }\">\r\n        <table class=\"expand-column-table\">\r\n          <tr>\r\n            <td>\r\n              <span (click)=\"expandRow(data.node_id, data)\" class=\"expand-icon-container\" title=\"Expand row\">\r\n                <i class=\"{{this.configs.css.expand_class}}\" aria-hidden=\"true\"></i>\r\n              </span>\r\n            </td>\r\n            <td>\r\n              <div class=\"parent-container\">\r\n                <db-tree-cell \r\n                [cell_value]=\"data.node_text\" \r\n                [row_data]=\"data\"\r\n                [column]=\"configs.group_by_column\">\r\n              </db-tree-cell>\r\n            </div>\r\n            </td>\r\n          </tr>\r\n        </table>\r\n    </td>\r\n    <td *ngIf=\"expand_tracker[data.node_id]\"  \r\n        [ngClass]=\"{\r\n          'clear-right-border': true, \r\n          'clear-left-border': (configs.actions.edit || configs.actions.add || configs.actions.delete)\r\n        }\"\r\n        >\r\n        <table class=\"expand-column-table\">\r\n          <tr>\r\n            <td>\r\n              <span (click)=\"collapseRow(data.node_id, data)\" class=\"expand-icon-container\" title=\"Collapse row\">\r\n                <i class=\"{{this.configs.css.collapse_class}}\" aria-hidden=\"true\"></i>\r\n              </span>\r\n            </td>\r\n            <td>\r\n              <div class=\"parent-container\">\r\n                <db-tree-cell \r\n                  [cell_value]=\"data.node_text\" \r\n                  [row_data]=\"data\"\r\n                  [column]=\"configs.group_by_column\">\r\n                </db-tree-cell>\r\n              </div>\r\n            </td>\r\n          </tr>\r\n        </table>\r\n    </td>\r\n    <!-- For parent levels add extra tds -->\r\n    <td \r\n      *ngFor=\"let level of range(configs.group_by.length - data.level + 1)\" \r\n      class=\"clear-right-border clear-left-border\">\r\n    </td>\r\n    <td *ngFor=\"let column of columns\" [ngClass]=\"{'column-hide': column.hidden}\">\r\n      <ng-container *ngIf=\"column.group_aggregator && data.last_parent\">\r\n        <span [innerHTML]=\"column.group_aggregator(fetchTraversedPaths(data.node_id))\"></span>\r\n      </ng-container>\r\n      <ng-container *ngIf=\"column.group_component && data.last_parent\">\r\n        <db-tree-cell \r\n          [cell_value]=\"data\" \r\n          [row_data]=\"fetchTraversedPaths(data.node_id)\" \r\n          [column]=\"column.group_component\">\r\n        </db-tree-cell>\r\n      </ng-container>\r\n    </td>\r\n  </ng-container>\r\n  <ng-container *ngIf=\"expand_tracker[data.parent_id] && !data.parent\">\r\n    <td *ngIf=\"configs.multi_select\" class=\"checkbox_column\" (click)=\"selectRowOnCheck(data, $event)\">\r\n      <input type=\"checkbox\" [checked]=\"data.row_selected\" *ngIf=\"showSelectBox(data)\">\r\n    </td>\r\n    <td db-tree-cell-actions \r\n      *ngIf=\"configs.actions.edit || configs.actions.delete || configs.actions.add\" \r\n      [data]=\"data\"\r\n      [configs]=\"configs\"\r\n      [store]=\"store\"\r\n      [edit_tracker]=\"edit_tracker\"\r\n      [rowdelete]=\"rowdelete\"\r\n      [internal_configs]=\"internal_configs\"\r\n      (canceledit)=\"cancelEdit($event)\" \r\n      (editcomplete)=\"saveRecord($event)\">\r\n    </td>\r\n    <td \r\n      [ngClass]=\"{\r\n        'clear-right-border': true, \r\n        'clear-left-border': (configs.actions.edit || configs.actions.add || configs.actions.delete)\r\n      }\" \r\n      *ngFor=\"let header of configs.group_by\">\r\n    </td>\r\n    <td class=\"clear-left-border\" *ngFor=\"let column of columns\" [ngClass]=\"{'column-hide': column.hidden}\">\r\n      <db-tree-cell \r\n        [cell_value]=\"data[column.name]\" \r\n        [row_data]=\"data\" \r\n        [column]=\"column\" \r\n        [cellclick]=\"cellclick\"\r\n        [edit_on]=\"edit_tracker[data['idx']]\"\r\n        (canceledit)=\"cancelEdit($event)\"\r\n        (editcomplete)=\"saveRecord($event)\">\r\n      </db-tree-cell>            \r\n    </td>\r\n  </ng-container>\r\n  \r\n</tr>\r\n",
                styles: [".db-tree-grid{border:1px solid grey;border-collapse:collapse;border-spacing:0;color:#4b4b4b;display:table;line-height:1.5em;max-width:100%;overflow:auto;word-break:normal;word-break:keep-all}select{padding:5px}tr.child{background:#fff}tr.parent{background:#fafbff}tr.selected{background-color:#e2e7eb}tr span.expand-icon-container{cursor:pointer;padding-right:10px}tr div.parent-container{display:inline-block}tr i.minus{border:1px solid transparent;margin-right:23px}tr i.minus:before{border-top:2px solid #222;content:\"\";display:block;height:1px;margin-left:6px;margin-top:10px;position:absolute;width:18px}tr td{border:1px solid #cdd5dc;padding:.5rem;position:relative;vertical-align:middle}tr td.checkbox_column{text-align:center}tr td:first-child{padding:.3rem}tr td i.plus{border:1px solid transparent;margin-right:23px}tr td i.plus:before{border-top:2px solid #222;height:1px;margin-left:6px;margin-top:10px;width:18px}tr td i.plus:after,tr td i.plus:before{content:\"\";display:block;position:absolute}tr td i.plus:after{border-right:2px solid #222;height:18px;margin-left:14px;margin-top:2px;width:1px}td.clear-left-border{border-left:0!important}td.clear-right-border{border-right:0!important}.column-hide{display:none}.expand-column-table td{border:none;padding:0!important}"]
            },] }
];
TreeBodyComponent.ctorParameters = () => [];
TreeBodyComponent.propDecorators = {
    store: [{ type: Input }],
    expand_tracker: [{ type: Input }],
    edit_tracker: [{ type: Input }],
    columns: [{ type: Input }],
    configs: [{ type: Input }],
    internal_configs: [{ type: Input }],
    rowdelete: [{ type: Input }],
    rowsave: [{ type: Input }],
    rowdeselect: [{ type: Input }],
    cellclick: [{ type: Input }],
    rowadd: [{ type: Output }],
    rowexpand: [{ type: Output }],
    rowcollapse: [{ type: Output }],
    rowselect: [{ type: Output }]
};

class TreeCellViewComponent {
    constructor() {
        this.cellclick = new EventEmitter();
    }
    ngOnInit() {
    }
}
TreeCellViewComponent.decorators = [
    { type: Component, args: [{
                selector: 'db-tree-cell-view',
                template: "<div (click)=\"cellclick.emit($event)\">{{column.renderer ? column.renderer(cell_value, row_data) : cell_value}}</div>",
                styles: [""]
            },] }
];
TreeCellViewComponent.ctorParameters = () => [];
TreeCellViewComponent.propDecorators = {
    cell_value: [{ type: Input }],
    row_data: [{ type: Input }],
    column: [{ type: Input }],
    cellclick: [{ type: Output }]
};

class CustomCellViewComponent {
    constructor(resolver) {
        this.resolver = resolver;
    }
    ngOnInit() {
        if (this.column.component && !this.custom_component) {
            this.createCustomComponent();
            this.callOnComponentInit();
        }
    }
    ngOnDestroy() {
        if (this.custom_component) {
            this.custom_component.destroy();
        }
    }
    createCustomComponent() {
        const componentFactory = this.resolver.resolveComponentFactory(this.column.component);
        this.custom_component = this.custom_view.createComponent(componentFactory);
    }
    callOnComponentInit() {
        this.column.on_component_init && this.column.on_component_init(this.custom_component.instance);
        this.custom_component.instance.cell_value = this.cell_value;
        this.custom_component.instance.row_data = this.row_data;
    }
}
CustomCellViewComponent.decorators = [
    { type: Component, args: [{
                selector: 'db-custom-cell-component',
                template: `
      <ng-template #customView></ng-template>
    `
            },] }
];
CustomCellViewComponent.ctorParameters = () => [
    { type: ComponentFactoryResolver }
];
CustomCellViewComponent.propDecorators = {
    column: [{ type: Input }],
    cell_value: [{ type: Input }],
    row_data: [{ type: Input }],
    custom_view: [{ type: ViewChild, args: ['customView', { read: ViewContainerRef, static: true },] }]
};

class DefaultEditor {
    constructor() {
        this.canceledit = new EventEmitter();
        this.editcomplete = new EventEmitter();
        this.cellclick = new EventEmitter();
    }
}
DefaultEditor.decorators = [
    { type: Component, args: [{
                selector: '',
                template: ''
            },] }
];
DefaultEditor.propDecorators = {
    canceledit: [{ type: Output }],
    editcomplete: [{ type: Output }],
    cellclick: [{ type: Output }]
};

class CustomCellEditorComponent extends DefaultEditor {
    constructor(resolver) {
        super();
        this.resolver = resolver;
    }
    ngOnInit() {
        if (this.column.editor && !this.custom_component) {
            this.createCustomComponent();
            this.callOnComponentInit();
        }
    }
    ngOnDestroy() {
        if (this.custom_component) {
            this.custom_component.destroy();
        }
    }
    createCustomComponent() {
        const componentFactory = this.resolver.resolveComponentFactory(this.column.editor);
        this.custom_component = this.custom_view.createComponent(componentFactory);
    }
    callOnComponentInit() {
        this.column.on_component_init && this.column.on_component_init(this.custom_component.instance);
        this.custom_component.instance.cell_value = this.cell_value;
        this.custom_component.instance.row_data = this.row_data;
        this.custom_component.instance.column = this.column;
        this.custom_component.instance.editcomplete.subscribe((event) => this.editcomplete.emit(event));
        this.custom_component.instance.canceledit.subscribe((event) => this.canceledit.emit(event));
        this.custom_component.instance.cellclick.subscribe((event) => this.cellclick.emit(event));
    }
}
CustomCellEditorComponent.decorators = [
    { type: Component, args: [{
                selector: 'db-custom-cell-editor-component',
                template: `
      <ng-template #customView></ng-template>
    `
            },] }
];
CustomCellEditorComponent.ctorParameters = () => [
    { type: ComponentFactoryResolver }
];
CustomCellEditorComponent.propDecorators = {
    column: [{ type: Input }],
    cell_value: [{ type: Input }],
    row_data: [{ type: Input }],
    custom_view: [{ type: ViewChild, args: ['customView', { read: ViewContainerRef, static: true },] }]
};

class TreeCellEditorComponent extends DefaultEditor {
    constructor() {
        super();
    }
    ngOnInit() {
    }
}
TreeCellEditorComponent.decorators = [
    { type: Component, args: [{
                selector: 'db-tree-cell-editor',
                template: "<input type=\"text\" \r\n    [(ngModel)]=\"row_data[column.name]\" \r\n    class=\"cell-default\" \r\n    (click)=\"cellclick.emit($event)\"\r\n    (keydown.enter)=\"editcomplete.emit($event)\"\r\n    (keydown.esc)=\"canceledit.emit()\"\r\n    >",
                styles: ["input.cell-default{padding:5px}"]
            },] }
];
TreeCellEditorComponent.ctorParameters = () => [];
TreeCellEditorComponent.propDecorators = {
    cell_value: [{ type: Input }],
    row_data: [{ type: Input }],
    column: [{ type: Input }]
};

class TreeCellActionsComponent {
    constructor() {
        this.editcomplete = new EventEmitter();
        this.canceledit = new EventEmitter();
    }
    ngOnInit() {
    }
    enableEdit(row_data) {
        const index = row_data['idx'];
        this.edit_tracker[index] = true;
        this.internal_configs.current_edited_row = Object.assign({}, row_data);
    }
    findRecordIndex(idx) {
        for (let index = 0; index < this.store.processed_data.length; index++) {
            if (this.store.processed_data[index].idx === idx) {
                return index;
            }
        }
    }
    deleteRecord(rec) {
        if (this.configs.actions.resolve_delete) {
            const promise = new Promise((resolve, reject) => {
                this.rowdelete.emit({
                    data: rec,
                    resolve: resolve
                });
            });
            promise.then(() => {
                this.store.processed_data.splice(this.findRecordIndex(rec.idx), 1);
                this.store.refreshDisplayData();
            }).catch((err) => { });
        }
        else {
            this.store.processed_data.splice(this.findRecordIndex(rec.idx), 1);
            this.store.refreshDisplayData();
            this.rowdelete.emit(rec);
        }
    }
    saveRecord($event) {
        this.editcomplete.emit({ event: $event, data: this.data });
    }
}
TreeCellActionsComponent.decorators = [
    { type: Component, args: [{
                selector: '[db-tree-cell-actions]',
                template: "<ng-container *ngIf=\"configs.actions.edit && configs.row_edit_function(data)\">\r\n  <div *ngIf=\"!edit_tracker[data['idx']]\" class=\"icon-container\">\r\n    <i (click)=\"enableEdit(data)\" *ngIf=\"configs.css.edit_class\" title=\"Edit row\" class=\"{{this.configs.css.edit_class}}\" aria-hidden=\"true\" ></i>\r\n    <svg (click)=\"enableEdit(data)\" *ngIf=\"!configs.css.edit_class\" xmlns=\"http://www.w3.org/2000/svg\" viewBox=\"0 0 24 24\"><title>Edit row</title><g data-name=\"Layer 2\"><g data-name=\"edit\"><rect width=\"24\" height=\"24\" opacity=\"0\"/><path d=\"M19.4 7.34L16.66 4.6A2 2 0 0 0 14 4.53l-9 9a2 2 0 0 0-.57 1.21L4 18.91a1 1 0 0 0 .29.8A1 1 0 0 0 5 20h.09l4.17-.38a2 2 0 0 0 1.21-.57l9-9a1.92 1.92 0 0 0-.07-2.71zM9.08 17.62l-3 .28.27-3L12 9.32l2.7 2.7zM16 10.68L13.32 8l1.95-2L18 8.73z\"/></g></g></svg>             \r\n  </div>\r\n  <ng-container *ngIf=\"edit_tracker[data['idx']]\">\r\n    <div class=\"icon-container\">\r\n      <i (click)=\"saveRecord($event)\" *ngIf=\"configs.css.save_class\" title=\"Save row\" class=\"{{this.configs.css.save_class}}\" aria-hidden=\"true\"></i>\r\n      <svg (click)=\"saveRecord($event)\" title=\"Save row\" *ngIf=\"!configs.css.save_class\" xmlns=\"http://www.w3.org/2000/svg\" viewBox=\"0 0 24 24\"><title>Save row</title><g data-name=\"Layer 2\"><g data-name=\"save\"><rect width=\"24\" height=\"24\" opacity=\"0\"/><path d=\"M20.12 8.71l-4.83-4.83A3 3 0 0 0 13.17 3H6a3 3 0 0 0-3 3v12a3 3 0 0 0 3 3h12a3 3 0 0 0 3-3v-7.17a3 3 0 0 0-.88-2.12zM10 19v-2h4v2zm9-1a1 1 0 0 1-1 1h-2v-3a1 1 0 0 0-1-1H9a1 1 0 0 0-1 1v3H6a1 1 0 0 1-1-1V6a1 1 0 0 1 1-1h2v5a1 1 0 0 0 1 1h4a1 1 0 0 0 0-2h-3V5h3.17a1.05 1.05 0 0 1 .71.29l4.83 4.83a1 1 0 0 1 .29.71z\"/></g></g></svg>\r\n    </div>\r\n    <div class=\"icon-container\">\r\n      <i (click)=\"canceledit.emit(data)\" *ngIf=\"configs.css.cancel_class\" title=\"Cancel row\" class=\"{{this.configs.css.cancel_class}}\" aria-hidden=\"true\"></i>\r\n      <svg (click)=\"canceledit.emit(data)\" *ngIf=\"!configs.css.cancel_class\" title=\"Cancel row\" xmlns=\"http://www.w3.org/2000/svg\" viewBox=\"0 0 24 24\"><title>Cancel row</title><g data-name=\"Layer 2\"><g data-name=\"close\"><rect width=\"24\" height=\"24\" transform=\"rotate(180 12 12)\" opacity=\"0\"/><path d=\"M13.41 12l4.3-4.29a1 1 0 1 0-1.42-1.42L12 10.59l-4.29-4.3a1 1 0 0 0-1.42 1.42l4.3 4.29-4.3 4.29a1 1 0 0 0 0 1.42 1 1 0 0 0 1.42 0l4.29-4.3 4.29 4.3a1 1 0 0 0 1.42 0 1 1 0 0 0 0-1.42z\"/></g></g></svg>\r\n    </div>\r\n  </ng-container>\r\n</ng-container>            \r\n<div *ngIf=\"configs.actions.delete && !edit_tracker[data['idx']] && configs.row_delete_function(data)\" class=\"icon-container\">\r\n  <i *ngIf=\"configs.css.delete_class\" title=\"Delete row\" (click)=\"deleteRecord(data)\" class=\"{{this.configs.css.delete_class}}\" aria-hidden=\"true\"></i>\r\n  <svg *ngIf=\"!configs.css.delete_class\" title=\"Delete row\" (click)=\"deleteRecord(data)\" xmlns=\"http://www.w3.org/2000/svg\" viewBox=\"0 0 24 24\"><title>Delete row</title><g data-name=\"Layer 2\"><g data-name=\"trash-2\"><rect width=\"24\" height=\"24\" opacity=\"0\"/><path d=\"M21 6h-5V4.33A2.42 2.42 0 0 0 13.5 2h-3A2.42 2.42 0 0 0 8 4.33V6H3a1 1 0 0 0 0 2h1v11a3 3 0 0 0 3 3h10a3 3 0 0 0 3-3V8h1a1 1 0 0 0 0-2zM10 4.33c0-.16.21-.33.5-.33h3c.29 0 .5.17.5.33V6h-4zM18 19a1 1 0 0 1-1 1H7a1 1 0 0 1-1-1V8h12z\"/><path d=\"M9 17a1 1 0 0 0 1-1v-4a1 1 0 0 0-2 0v4a1 1 0 0 0 1 1z\"/><path d=\"M15 17a1 1 0 0 0 1-1v-4a1 1 0 0 0-2 0v4a1 1 0 0 0 1 1z\"/></g></g></svg>            \r\n</div>",
                styles: ["svg{padding-right:4px;width:25px}div.icon-container{color:#000;cursor:pointer;display:inline-block;text-align:center;width:50%}"]
            },] }
];
TreeCellActionsComponent.ctorParameters = () => [];
TreeCellActionsComponent.propDecorators = {
    store: [{ type: Input }],
    edit_tracker: [{ type: Input }],
    configs: [{ type: Input }],
    rowdelete: [{ type: Input }],
    data: [{ type: Input }],
    internal_configs: [{ type: Input }],
    editcomplete: [{ type: Output }],
    canceledit: [{ type: Output }]
};

class TreeCellComponent {
    constructor() {
        this.canceledit = new EventEmitter();
        this.editcomplete = new EventEmitter();
    }
    ngOnInit() {
    }
    onCellClick(rec, column) {
        this.cellclick.emit({ row: rec, column: column });
    }
    onEditComplete($event) {
        this.editcomplete.emit({ event: $event, data: this.row_data });
    }
}
TreeCellComponent.decorators = [
    { type: Component, args: [{
                selector: 'db-tree-cell',
                template: "<ng-container>\r\n    <div [ngSwitch]=\"column.type\" *ngIf=\"!edit_on || !column.editable\">\r\n        <db-custom-cell-component \r\n            *ngSwitchCase=\"'custom'\" \r\n            [cell_value]=\"cell_value\" \r\n            [column]=\"column\" \r\n            [row_data]=\"row_data\" \r\n            (click)=\"onCellClick(row_data, column)\">\r\n        </db-custom-cell-component>\r\n        <db-tree-cell-view \r\n            *ngSwitchDefault \r\n            (cellclick)=\"onCellClick(row_data, column)\"\r\n            [cell_value]=\"cell_value\" \r\n            [column]=\"column\" \r\n            [row_data]=\"row_data\">\r\n        </db-tree-cell-view>\r\n    </div>\r\n    <div *ngIf=\"edit_on && column.editable\">\r\n        <db-custom-cell-editor-component \r\n            *ngIf=\"column.editor\" \r\n            [cell_value]=\"cell_value\" \r\n            [column]=\"column\" \r\n            [row_data]=\"row_data\"\r\n            (canceledit)=\"canceledit.emit(row_data['idx'])\"\r\n            (editcomplete)=\"onEditComplete($event)\">\r\n        </db-custom-cell-editor-component>\r\n        <db-tree-cell-editor \r\n            *ngIf=\"!column.editor\"\r\n            [cell_value]=\"cell_value\" \r\n            [row_data]=\"row_data\" \r\n            [column]=\"column\"\r\n            (canceledit)=\"canceledit.emit(row_data['idx'])\"\r\n            (editcomplete)=\"onEditComplete($event)\"\r\n            >\r\n        </db-tree-cell-editor>\r\n    </div>\r\n</ng-container>\r\n"
            },] }
];
TreeCellComponent.ctorParameters = () => [];
TreeCellComponent.propDecorators = {
    cell_value: [{ type: Input }],
    row_data: [{ type: Input }],
    column: [{ type: Input }],
    edit_on: [{ type: Input }],
    cellclick: [{ type: Input }],
    canceledit: [{ type: Output }],
    editcomplete: [{ type: Output }]
};

class CellModule {
}
CellModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    TreeCellComponent,
                    TreeCellViewComponent,
                    CustomCellViewComponent,
                    CustomCellEditorComponent,
                    TreeCellEditorComponent,
                    TreeCellActionsComponent
                ],
                imports: [
                    CommonModule,
                    FormsModule
                ],
                exports: [
                    TreeCellComponent,
                    CustomCellViewComponent,
                    CustomCellEditorComponent,
                    TreeCellEditorComponent,
                    TreeCellActionsComponent
                ]
            },] }
];

class AddRowComponent {
    constructor() {
        this.row_data = {};
        this.rowadd = new EventEmitter();
        this.canceledit = new EventEmitter();
    }
    ngOnInit() {
        this.columns.forEach(column => {
            this.row_data[column.name] = '';
        });
    }
    saveAddRecord(e) {
        // const add_column = {};
        const index = this.store.processed_data.length;
        // this.columns.forEach(column => {
        //   if (column.editable) {
        //     add_column[column.name] = (document.getElementById(index + column.name) as HTMLInputElement).value;
        //   } else {
        //     add_column[column.name] = '';
        //   }
        // });
        this.configs.group_by.forEach(key => {
            this.row_data[key] = document.getElementById(index + key).value;
        });
        this.store.raw_data.push(this.row_data);
        this.internal_configs.show_add_row = false;
        this.rowadd.emit(this.row_data);
    }
    cancelAddEdit() {
        this.internal_configs.show_add_row = false;
    }
}
AddRowComponent.decorators = [
    { type: Component, args: [{
                selector: '[db-add-row]',
                template: "<td *ngIf=\"configs.multi_select\"></td>\r\n<td>\r\n  <div class=\"icon-container\">\r\n    <i (click)=\"saveAddRecord($event)\" *ngIf=\"configs.css.save_class\" title=\"Save row\" class=\"{{this.configs.css.save_class}}\" aria-hidden=\"true\"></i>\r\n    <svg (click)=\"saveAddRecord($event)\" title=\"Save row\" *ngIf=\"!configs.css.save_class\" xmlns=\"http://www.w3.org/2000/svg\" viewBox=\"0 0 24 24\"><title>Save row</title><g data-name=\"Layer 2\"><g data-name=\"save\"><rect width=\"24\" height=\"24\" opacity=\"0\"/><path d=\"M20.12 8.71l-4.83-4.83A3 3 0 0 0 13.17 3H6a3 3 0 0 0-3 3v12a3 3 0 0 0 3 3h12a3 3 0 0 0 3-3v-7.17a3 3 0 0 0-.88-2.12zM10 19v-2h4v2zm9-1a1 1 0 0 1-1 1h-2v-3a1 1 0 0 0-1-1H9a1 1 0 0 0-1 1v3H6a1 1 0 0 1-1-1V6a1 1 0 0 1 1-1h2v5a1 1 0 0 0 1 1h4a1 1 0 0 0 0-2h-3V5h3.17a1.05 1.05 0 0 1 .71.29l4.83 4.83a1 1 0 0 1 .29.71z\"/></g></g></svg>\r\n  </div>\r\n  <div class=\"icon-container\">\r\n    <i (click)=\"cancelAddEdit()\" *ngIf=\"configs.css.cancel_class\" title=\"Cancel row\" class=\"{{this.configs.css.cancel_class}}\" aria-hidden=\"true\"></i>\r\n    <svg (click)=\"cancelAddEdit()\" *ngIf=\"!configs.css.cancel_class\" title=\"Cancel row\" xmlns=\"http://www.w3.org/2000/svg\" viewBox=\"0 0 24 24\"><title>Cancel row</title><g data-name=\"Layer 2\"><g data-name=\"close\"><rect width=\"24\" height=\"24\" transform=\"rotate(180 12 12)\" opacity=\"0\"/><path d=\"M13.41 12l4.3-4.29a1 1 0 1 0-1.42-1.42L12 10.59l-4.29-4.3a1 1 0 0 0-1.42 1.42l4.3 4.29-4.3 4.29a1 1 0 0 0 0 1.42 1 1 0 0 0 1.42 0l4.29-4.3 4.29 4.3a1 1 0 0 0 1.42 0 1 1 0 0 0 0-1.42z\"/></g></g></svg>\r\n  </div>\r\n</td>\r\n<td *ngFor=\"let group_by of configs.group_by\">\r\n  <select [attr.id]=\"store.processed_data.length+group_by\">\r\n    <option *ngFor=\"let key of store.group_keys[group_by]\">\r\n    {{key}}\r\n    </option>\r\n  </select>\r\n</td>\r\n<td *ngFor=\"let column of columns\" [ngClass]=\"{'column-hide': column.hidden}\" >\r\n  <div *ngIf=\"column.editable\">\r\n    <db-custom-cell-editor-component \r\n        *ngIf=\"column.editor\"\r\n        [cell_value]=\"\" \r\n        [column]=\"column\" \r\n        [row_data]=\"row_data\"\r\n        (canceledit)=\"cancelAddEdit()\"\r\n        (editcomplete)=\"saveAddRecord($event)\">\r\n    </db-custom-cell-editor-component>\r\n    <db-tree-cell-editor \r\n        *ngIf=\"!column.editor\"\r\n        [cell_value]=\"\" \r\n        [row_data]=\"row_data\" \r\n        [column]=\"column\"\r\n        (canceledit)=\"cancelAddEdit()\"\r\n        (editcomplete)=\"saveAddRecord($event)\"\r\n        >\r\n    </db-tree-cell-editor>\r\n  </div>\r\n</td>",
                encapsulation: ViewEncapsulation.None,
                styles: [".db-tree-grid{border:1px solid grey;border-collapse:collapse;border-spacing:0;color:#4b4b4b;display:table;line-height:1.5em;max-width:100%;overflow:auto;word-break:normal;word-break:keep-all}select{padding:5px}tr.child{background:#fff}tr.parent{background:#fafbff}tr.selected{background-color:#e2e7eb}tr span.expand-icon-container{cursor:pointer;padding-right:10px}tr div.parent-container{display:inline-block}tr i.minus{border:1px solid transparent;margin-right:23px}tr i.minus:before{border-top:2px solid #222;content:\"\";display:block;height:1px;margin-left:6px;margin-top:10px;position:absolute;width:18px}tr td{border:1px solid #cdd5dc;padding:.5rem;position:relative;vertical-align:middle}tr td.checkbox_column{text-align:center}tr td:first-child{padding:.3rem}tr td i.plus{border:1px solid transparent;margin-right:23px}tr td i.plus:before{border-top:2px solid #222;height:1px;margin-left:6px;margin-top:10px;width:18px}tr td i.plus:after,tr td i.plus:before{content:\"\";display:block;position:absolute}tr td i.plus:after{border-right:2px solid #222;height:18px;margin-left:14px;margin-top:2px;width:1px}td.clear-left-border{border-left:0!important}td.clear-right-border{border-right:0!important}.column-hide{display:none}.expand-column-table td{border:none;padding:0!important}svg{padding-right:4px;width:25px}div.icon-container{color:#000;display:inline-block;text-align:center;width:50%}"]
            },] }
];
AddRowComponent.ctorParameters = () => [];
AddRowComponent.propDecorators = {
    store: [{ type: Input }],
    columns: [{ type: Input }],
    configs: [{ type: Input }],
    internal_configs: [{ type: Input }],
    rowadd: [{ type: Output }],
    canceledit: [{ type: Output }]
};

class FilterRowComponent {
    constructor(ngTreeGridService) {
        this.ngTreeGridService = ngTreeGridService;
        this.search_values = {};
    }
    ngOnInit() {
        this.columns.forEach(column => {
            this.search_values[column.name] = '';
        });
    }
    filter() {
        this.store.filterBy(this.columns.map(column => column.name), Object.values(this.search_values));
        this.ngTreeGridService.expandAll(this.expand_tracker);
    }
}
FilterRowComponent.decorators = [
    { type: Component, args: [{
                selector: '[db-filter-row]',
                template: "<td *ngIf=\"configs.multi_select\"></td>\r\n<td *ngIf=\"(configs.actions.edit || configs.actions.delete || configs.actions.add)\"></td>\r\n<td *ngFor=\"let group_by of configs.group_by\"></td>\r\n<td *ngFor=\"let column of columns\">\r\n  <div *ngIf=\"column.filter\">\r\n    <input type=\"text\" (keyup)=\"filter()\" [(ngModel)]=\"search_values[column.name]\">\r\n  </div>\r\n</td>",
                styles: [".db-tree-grid{border:1px solid grey;border-collapse:collapse;border-spacing:0;color:#4b4b4b;display:table;line-height:1.5em;max-width:100%;overflow:auto;word-break:normal;word-break:keep-all}select{padding:5px}tr.child{background:#fff}tr.parent{background:#fafbff}tr.selected{background-color:#e2e7eb}tr span.expand-icon-container{cursor:pointer;padding-right:10px}tr div.parent-container{display:inline-block}tr i.minus{border:1px solid transparent;margin-right:23px}tr i.minus:before{border-top:2px solid #222;content:\"\";display:block;height:1px;margin-left:6px;margin-top:10px;position:absolute;width:18px}tr td{border:1px solid #cdd5dc;padding:.5rem;position:relative;vertical-align:middle}tr td.checkbox_column{text-align:center}tr td:first-child{padding:.3rem}tr td i.plus{border:1px solid transparent;margin-right:23px}tr td i.plus:before{border-top:2px solid #222;height:1px;margin-left:6px;margin-top:10px;width:18px}tr td i.plus:after,tr td i.plus:before{content:\"\";display:block;position:absolute}tr td i.plus:after{border-right:2px solid #222;height:18px;margin-left:14px;margin-top:2px;width:1px}td.clear-left-border{border-left:0!important}td.clear-right-border{border-right:0!important}.column-hide{display:none}.expand-column-table td{border:none;padding:0!important}td{border:1px solid #cdd5dc;padding:.2rem;position:relative;vertical-align:middle}td input{-moz-box-sizing:border-box;-webkit-box-sizing:border-box;box-sizing:border-box;padding:.1rem;width:100%}"]
            },] }
];
FilterRowComponent.ctorParameters = () => [
    { type: NgtreegridService }
];
FilterRowComponent.propDecorators = {
    store: [{ type: Input }],
    columns: [{ type: Input }],
    expand_tracker: [{ type: Input }],
    configs: [{ type: Input }],
    internal_configs: [{ type: Input }]
};

class BodyModule {
}
BodyModule.decorators = [
    { type: NgModule, args: [{
                declarations: [TreeBodyComponent, AddRowComponent, FilterRowComponent],
                imports: [
                    CommonModule,
                    CellModule,
                    FormsModule
                ],
                exports: [TreeBodyComponent, AddRowComponent]
            },] }
];

class TreeHeadComponent {
    constructor() {
        this.sortcolumn = new EventEmitter();
    }
    ngOnInit() { }
    addRow() {
        this.internal_configs.show_add_row = true;
    }
    sortColumn(column) {
        this.sortcolumn.emit(column);
    }
    selectAll(e) {
        if (e.target.checked) {
            this.store.selectAll();
            this.rowselectall.emit(e);
        }
        else {
            this.store.deSelectAll();
            this.rowdeselectall.emit(e);
        }
    }
}
TreeHeadComponent.decorators = [
    { type: Component, args: [{
                selector: '[db-tree-head]',
                template: "<tr [ngClass]=\"configs.css.header_class\">\r\n  <th *ngIf=\"configs.multi_select\" class=\"checkbox_column\"  [attr.width]=\"configs.multi_select_width\">\r\n    <input type=\"checkbox\" (click)=\"selectAll($event)\" [checked]=\"this.internal_configs.all_selected\">\r\n  </th>\r\n  <th \r\n    *ngIf=\"configs.actions.add || configs.actions.edit || configs.actions.delete\" \r\n    class=\"action-column\"\r\n    width=\"{{configs.action_column_width}}\"\r\n    (click)=\"addRow()\">\r\n    <span class=\"icon-container\">\r\n      <i *ngIf=\"!internal_configs.show_add_row && configs.actions.add\" [ngClass]=\"configs.css.add_class\" aria-hidden=\"true\" title=\"Add a row\"></i>\r\n      <span *ngIf=\"internal_configs.show_add_row || !configs.actions.add\">Actions</span>\r\n    </span>\r\n  </th>\r\n  <th class=\"clear-right-border group-by-column\" [attr.width]=\"configs.group_by_width[i]\"\r\n      *ngFor=\"let header of configs.group_by_header; let i = index\">\r\n    {{header}}\r\n  </th>\r\n  <th *ngFor=\"let column of columns\" (click)=\"sortColumn(column)\" [attr.width]=\"column.width\"\r\n    [ngClass]=\"{'sorted': column.sorted,'sortable': column.sortable, 'column-hide': column.hidden}\" >\r\n      <span>{{column.header}}</span>\r\n      <span *ngIf=\"column.sortable\" [ngClass]=\"column.sorted && column.sort_type == 0?'arrow-down active':'arrow-down'\"></span>\r\n      <span *ngIf=\"column.sortable\" [ngClass]=\"column.sorted && column.sort_type == 1?'arrow-up active':'arrow-up'\"></span>\r\n  </th>\r\n</tr>",
                styles: [".db-tree-grid{border:1px solid grey;border-collapse:collapse;border-spacing:0;color:#4b4b4b;display:table;line-height:1.5em;max-width:100%;overflow:auto;word-break:normal;word-break:keep-all}tr{background:#fff}tr th{color:#181818;font-weight:600;line-height:1.25}tr th.action-column span.icon-container,tr th.sortable{cursor:pointer}tr th.sortable:hover span.arrow-up{border-bottom:6px solid #afafaf}tr th.sortable:hover span.arrow-down{border-top:6px solid #afafaf}tr th span.arrow-up{border-bottom:6px solid #ddd;border-left:5px solid transparent;border-right:5px solid transparent;font-size:1rem;height:0;position:relative;top:-18px;width:0}tr th span.arrow-up.active{border-bottom:6px solid #8a8989!important}tr th span.arrow-down{border-left:5px solid transparent;border-right:5px solid transparent;border-top:6px solid #ddd;font-size:1rem;height:0;left:10px;position:relative;top:17px;width:0}tr th span.arrow-down.active{border-top:6px solid #8a8989!important}tr th{border:1px solid #cdd5dc;padding:.5rem;position:relative;vertical-align:middle}tr th:first-child{padding:.3rem}tr th i.plus{border:1px solid transparent}tr th i.plus:before{border-top:2px solid #222;height:1px;margin-left:6px;margin-top:-15px;width:18px}tr th i.plus:after,tr th i.plus:before{content:\"\";display:block;position:absolute}tr th i.plus:after{border-right:2px solid #222;height:18px;margin-left:14px;margin-top:-23px;width:1px}tr th.clear-left-border{border-left:0!important}tr th.clear-right-border{border-right:0!important}tr th.checkbox_column{text-align:center}.column-hide{display:none}svg{padding-right:5px;width:25px}"]
            },] }
];
TreeHeadComponent.ctorParameters = () => [];
TreeHeadComponent.propDecorators = {
    store: [{ type: Input }],
    columns: [{ type: Input }],
    configs: [{ type: Input }],
    internal_configs: [{ type: Input }],
    sortcolumn: [{ type: Output }],
    rowselectall: [{ type: Input }],
    rowdeselectall: [{ type: Input }]
};

class HeadModule {
}
HeadModule.decorators = [
    { type: NgModule, args: [{
                declarations: [TreeHeadComponent],
                imports: [
                    CommonModule
                ],
                exports: [TreeHeadComponent]
            },] }
];

class NgtreegridModule {
}
NgtreegridModule.decorators = [
    { type: NgModule, args: [{
                declarations: [NgtreegridComponent],
                imports: [
                    CommonModule,
                    BodyModule,
                    HeadModule
                ],
                exports: [NgtreegridComponent]
            },] }
];

/*
 * Public API Surface of ngtreegrid
 */

/**
 * Generated bundle index. Do not edit.
 */

export { DefaultEditor, NgtreegridComponent, NgtreegridModule, NgtreegridService, BodyModule as ɵa, TreeBodyComponent as ɵb, AddRowComponent as ɵc, FilterRowComponent as ɵd, CellModule as ɵe, TreeCellComponent as ɵf, TreeCellViewComponent as ɵg, CustomCellViewComponent as ɵh, CustomCellEditorComponent as ɵi, TreeCellEditorComponent as ɵj, TreeCellActionsComponent as ɵk, HeadModule as ɵl, TreeHeadComponent as ɵm };
//# sourceMappingURL=ngtreegrid.js.map
